import 'package:args/args.dart';
import 'package:comet_cli/cmd_build.dart';
import 'package:comet_cli/comet_yaml.dart';
import 'package:comet_cli/flutter_build.dart';
import 'package:comet_cli/gh_pages.dart';
import 'package:comet_cli/run_deploy_help.dart';

Future<void> cmdDeploy(
  ArgParser parser,
  List<String> arguments,
) async {
  final deployArgsMap = parser.parse(arguments);
  final showHelp = deployArgsMap['help'] as bool;
  if (showHelp) {
    await runDeployHelp();
    return;
  }

  // Cometプロジェクトかどうか
  final cometYaml = readCometYaml();
  if (cometYaml == null) {
    print('このフォルダはcometプロジェクトではありません');
    return;
  }

  final githubUserId = cometYaml.github.userId;
  final githubRepoName = cometYaml.github.repoName;
  if (githubUserId == null) {
    print('github.user_id が設定されていません');
    return;
  }

  if (githubRepoName == null) {
    print('github.repo_name が設定されていません');
    return;
  }

  // comet build
  await executeBuild();

  // Flutter ビルド
  await flutterBuild();

  // Git リポジトリかどうか
  final ghPages = GHPages();
  final isGitdir = await ghPages.isGitDir();

  if (!isGitdir) {
    // Gitリポジトリではないのでinit
    print('git init...');
    await ghPages.gitInit();
  }

  // リモートリポジトリを持っているか
  final hasRemote = await ghPages.hasRemote();

  if (!hasRemote) {
    // リモートリポジトリを持っていないのでadd
    print('git remote add origin...');
    await ghPages.addOrigin(
      githubUserId,
      githubRepoName,
    );
  }

  // gh-pages ブランチにコミット
  await ghPages.commitGHBranch();

  // gh-pages ブランチをプッシュ
  await ghPages.pushGHBranch();

  print('完了 以下のURLにサイトが公開されました (数分かかります)');
  print('https://$githubUserId.github.io/$githubRepoName/');
}
