import 'package:args/args.dart';
import 'package:comet_cli/cmd_build.dart';
import 'package:comet_cli/cmd_create.dart';
import 'package:comet_cli/cmd_deploy.dart';
import 'package:comet_cli/cmd_doctor.dart';
import 'package:comet_cli/cmd_start.dart';
import 'package:comet_cli/run_help.dart';
import 'package:comet_cli/run_version.dart';

Future<void> cmdRoot(List<String> arguments) async {
  final parser = ArgParser();
  // --help, -h を登録
  parser.addFlag('help', abbr: 'h', help: 'Show usage information');
  // --version, -v を登録
  parser.addFlag('version', abbr: 'v', help: 'Show version number');

  // サブコマンド deploy を登録
  final deployParser = parser.addCommand('deploy');
  // deploy に --help, -h を登録
  deployParser.addFlag('help', abbr: 'h', help: 'Show usage information');

  // サブコマンド create を登録
  final createParser = parser.addCommand('create');
  // create に --help, -h を登録
  createParser.addFlag('help', abbr: 'h', help: 'Show usage information');

  // サブコマンド doctor を登録
  final doctorParser = parser.addCommand('doctor');
  // doctor に --help, -h を登録
  doctorParser.addFlag('help', abbr: 'h', help: 'Show usage information');

  // サブコマンド start を登録
  final startParser = parser.addCommand('start');
  // start に --help, -h を登録
  startParser.addFlag('help', abbr: 'h', help: 'Show usage information');

  // サブコマンド build を登録
  final buildParser = parser.addCommand('build');
  // build に --help, -h を登録
  buildParser.addFlag('help', abbr: 'h', help: 'Show usage information');

  /* START PARSE */

  final argsMap = parser.parse(arguments);

  final showHelp = argsMap['help'] as bool;
  if (showHelp) {
    await runHelp();
    return;
  }
  final showVersion = argsMap['version'] as bool;
  if (showVersion) {
    await runVersion();
    return;
  }

  final subCommand = argsMap.command;
  if (subCommand == null) {
    // サブコマンドがなければここで終了
    return;
  }

  switch (subCommand.name) {
    case 'deploy':
      await cmdDeploy(deployParser, subCommand.arguments);
      break;
    case 'create':
      await cmdCreate(createParser, subCommand.arguments);
      break;
    case 'doctor':
      await cmdDoctor(doctorParser, subCommand.arguments);
      break;
    case 'start':
      await cmdStart(startParser, subCommand.arguments);
      break;
    case 'build':
      await cmdBuild(buildParser, subCommand.arguments);
      break;
    default:
      break;
  }
}
