import 'dart:io';
import 'package:comet_cli/config.dart';
import 'package:comet_cli/create_commit_msg.dart';
import 'package:git/git.dart';
import 'package:path/path.dart' as path;

class GHPages {
  final String branch = githubPagesBranch;
  GitDir? _gitDirCache;

  Future<GitDir> get _gitDir async {
    _gitDirCache ??= await GitDir.fromExisting(
      path.current,
      allowSubdirectory: true,
    );
    return _gitDirCache!;
  }

  Future<bool> isGitDir() async {
    return GitDir.isGitDir(path.current);
  }

  Future<void> gitInit() async {
    await GitDir.init(path.current, initialBranch: 'main');
  }

  Future<void> addOrigin(String userId, String repoName) async {
    await Process.run(
      'git',
      [
        'remote',
        'add',
        'origin',
        'https://github.com/$userId/$repoName.git',
      ],
      runInShell: true,
    );
  }

  Future<void> commitGHBranch() async {
    final indexPath = flutterBuildWebDist;
    final indexDir = path.join(path.current, indexPath);
    final msg = createCommitMsg();
    final gitDir = await _gitDir;
    await gitDir.updateBranchWithDirectoryContents(
      branch,
      indexDir,
      msg,
    );
    print('comit ok, branch: $branch');
  }

  Future<void> pushGHBranch({
    String remote = 'origin',
  }) async {
    // Push
    final gitDir = await _gitDir;
    await gitDir.runCommand(['push', remote, branch]);
  }

  Future<bool> hasRemote() async {
    // check remote
    final gitDir = await _gitDir;
    final result = await gitDir.runCommand(['remote']);
    final exception = Exception('origin not found');
    if (result.exitCode != 0) throw exception;
    if (result.stdout is! String) throw exception;
    final remotes = result.stdout as String;
    final remoteExists = remotes.isNotEmpty;
    return remoteExists;
  }
}
